@@ -34,6 +34,7 @@ import itertools
 import unicodedata
 import aiohttp
 
+from xml.etree import ElementTree
 from collections import defaultdict
 from typing import TYPE_CHECKING, Optional, Union, Awaitable, Any, Tuple
 
@@ -68,9 +69,34 @@ class EventContext:
 class EventDispatcher:
     def __init__(self) -> None:
         self._listeners = defaultdict(list)
+        self._presence_listeners = []
         self.interactions_enabled = False
 
-    def process_event(self, client: 'Client', body: dict) -> None:
+    def process_presence(self, client, *args) -> None:
+        for coro in self._presence_listeners:
+            if __name__ == coro.__module__:
+                asyncio.ensure_future(coro(client.xmpp, *args))
+            else:
+                asyncio.ensure_future(coro(*args))
+
+    def presence(self) -> Awaitable:
+        def decorator(coro: Awaitable) -> Awaitable:
+            self.add_presence_handler(coro)
+            return coro
+        return decorator
+
+    def add_presence_handler(self, coro: Awaitable) -> None:
+        if coro not in self._presence_listeners:
+            self._presence_listeners.append(coro)
+
+    def remove_presence_handler(self, coro: Awaitable) -> None:
+        self._presence_listeners = [
+            c for c in self._presence_listeners if c is not coro
+        ]
+
+    def process_event(self, client: 'Client', raw_body: dict) -> None:
+        body = json.loads(raw_body)
+
         type_ = body.get('type')
         if type_ is None:
             if self.interactions_enabled:
@@ -111,44 +137,159 @@ class EventDispatcher:
 dispatcher = EventDispatcher()
 
 
+class XMLProcessor:
+    def _process_presence(self, raw: str) -> Optional[Union[tuple, bool]]:
+        tree = ElementTree.fromstring(raw)
+
+        type_ = tree.get('type')
+
+        # Only intercept presences with either no type attribute
+        # (which means available) or unavailable type.
+        if type_ is not None and type_ not in ('available', 'unavailable'):
+            return False
+
+        from_ = tree.get('from')
+
+        # If from is a party, let aioxmpp handle it.
+        if from_ is not None and '-' in from_:
+            return False
+
+        status = None
+        show = None
+        for elem in tree:
+            if 'status' in elem.tag:
+                status = elem.text
+            if 'show' in elem.tag:
+                show = elem.text
+
+        # We have no use for the presence if status is None and
+        # therefore it's better to just let aioxmpp handle it.
+        if status is None:
+            return False
+
+        split = from_.split('@')
+        user_id = split[0]
+        platform = split[1].split(':')[2]
+
+        return ('presence', user_id, platform, type_, status, show)
+
+    def _process_message(self, raw: str) -> Optional[Union[tuple, bool]]:
+        tree = ElementTree.fromstring(raw)
+
+        type_ = tree.get('type')
+
+        # Only intercept messages with either no type attribute
+        # (which means normal) or  type.
+        if type_ is not None and type_ != 'normal':
+            return False
+
+        # Let aioxmpp handle it if no body tag is found. Also, technically
+        # a message can include multiple body tags for different languages
+        # but afaik only one body tag is sent from epics servers.
+        body = None
+        for elem in tree:
+            if 'body' in elem.tag:
+                body = elem.text
+                break
+        else:
+            return False
+
+        return ('message', body)
+
+    def process(self, raw: str) -> Optional[Union[tuple, bool]]:
+        # Yes, this is a hacky solution but it's better than
+        # using the quite unnecessary slow aioxmpp one.
+        if '<presence' in raw:
+            return self._process_presence(raw)
+        elif '<message' in raw:
+            return self._process_message(raw)
+
+        return False
+
+
 class WebsocketTransport:
     def __init__(self, loop: asyncio.AbstractEventLoop,
                  stream: 'WebsocketXMLStream',
-                 logger: logging.Logger) -> None:
+                 client: 'Client',
+                 logger: logging.Logger,
+                 ws_connector: Optional[aiohttp.BaseConnector] = None) -> None:
         self.loop = loop
         self.stream = stream
+        self.client = client
         self.logger = logger
+        self.ws_connector = ws_connector
+
+        self.xml_processor = XMLProcessor()
 
         self.connection = None
         self._buffer = b''
         self._reader_task = None
         self._close_event = asyncio.Event()
+        self._called_lost = False
 
     async def create_connection(self, *args,
                                 **kwargs) -> aiohttp.ClientWebSocketResponse:
-        self.session = aiohttp.ClientSession()
+        self.logger.debug('Setting up new websocket connection.')
+
+        self.session = aiohttp.ClientSession(
+            connector=self.ws_connector,
+            connector_owner=self.ws_connector is None,
+        )
         self.connection = con = await self.session.ws_connect(
             *args, **kwargs
         )
+
         self.loop.create_task(self.reader())
         self.stream.connection_made(self)
+        self._called_lost = False
+
         self.logger.debug('Websocket connection established.')
         return con
 
     async def reader(self) -> None:
         self.logger.debug('Websocket reader is now running.')
 
-        while True:
-            msg = await self.connection.receive()
+        try:
+            async for msg in self.connection:
+                self.logger.debug('RECV: {0}'.format(msg))
+                if msg.type == aiohttp.WSMsgType.TEXT:
+                    ret = self.xml_processor.process(msg.data)
+                    if ret is None:
+                        continue
+                    elif ret is False:
+                        self.stream.data_received(msg.data)
+                    else:
+                        type_ = ret[0]
+                        if type_ == 'presence':
+                            dispatcher.process_presence(
+                                self.client,
+                                *ret[1:]
+                            )
+                        elif type_ == 'message':
+                            dispatcher.process_event(
+                                self.client,
+                                *ret[1:]
+                            )
 
-            if msg.type == aiohttp.WSMsgType.text:
-                self.stream.data_received(msg.data)
-            if msg.type == aiohttp.WSMsgType.closed:
-                break
-            if msg.type == aiohttp.WSMsgType.error:
-                break
+                if msg.type == aiohttp.WSMsgType.CLOSED:
+                    self._called_lost = True
+                    self.stream.connection_lost(
+                        ConnectionError('websocket stream closed by peer')
+                    )
+                    break
+                if msg.type == aiohttp.WSMsgType.ERROR:
+                    self._called_lost = True
+                    self.stream.connection_lost(
+                        ConnectionError(
+                            'websocket stream received an error: '
+                            '{0}'.format(self.connection.exception()))
+                    )
+                    break
+        finally:
+            self.logger.debug('Websocket reader stopped.')
 
     async def send(self, data: bytes) -> None:
+        self.logger.debug('SEND: {0}'.format(data))
         await self.connection.send_bytes(data)
 
     def write(self, data: bytes) -> None:
@@ -164,7 +305,7 @@ class WebsocketTransport:
         return False
 
     def write_eof(self) -> None:
-        raise NotImplementedError("Cannot write_eof() on ws transport")
+        raise NotImplementedError("Cannot write_eof() on ws transport.")
 
     def _stop_reader(self) -> None:
         if self._reader_task is not None and not self._reader_task.cancelled():
@@ -174,11 +315,9 @@ class WebsocketTransport:
         self._close_event.set()
 
     def on_close(self, *args) -> None:
-        # We do this to make sure connection_lost() doesnt re-call
-        # _writer.abort().
-        self.stream._writer = None
-
-        self.stream.connection_lost(None)
+        if not self._called_lost:
+            self._called_lost = True
+            self.stream.connection_lost(None)
 
         try:
             task = self.loop.create_task(self.session.close())
@@ -191,11 +330,17 @@ class WebsocketTransport:
         if not self.connection:
             raise RuntimeError('Cannot close a non-existing connection.')
 
+        self.logger.debug('Closing websocket connection.')
+
         task = self.loop.create_task(self.connection.close())
         task.add_done_callback(self.on_close)
 
         self._stop_reader()
 
+    def abort(self) -> None:
+        self.logger.debug('Received abort signal.')
+        self.close()
+
     async def wait_closed(self) -> None:
         await self._close_event
 
@@ -246,6 +391,10 @@ class WebsocketXMLStream(aioxmpp.protocol.XMLStream):
 
 
 class XMPPOverWebsocketConnector(aioxmpp.connector.BaseConnector):
+    def __init__(self, client, ws_connector=None):
+        self.client = client
+        self.ws_connector = ws_connector
+
     @property
     def tls_supported(self) -> bool:
         return False
@@ -282,7 +431,9 @@ class XMPPOverWebsocketConnector(aioxmpp.connector.BaseConnector):
         transport = WebsocketTransport(
             loop,
             stream,
+            self.client,
             logger,
+            ws_connector=self.ws_connector
         )
         await transport.create_connection(
             'wss://{host}'.format(host=host),
@@ -294,8 +445,9 @@ class XMPPOverWebsocketConnector(aioxmpp.connector.BaseConnector):
 
 
 class XMPPClient:
-    def __init__(self, client: 'Client') -> None:
+    def __init__(self, client: 'Client', ws_connector=None) -> None:
         self.client = client
+        self.ws_connector = ws_connector
 
         self.xmpp_client = None
         self.stream = None
@@ -543,10 +695,10 @@ class XMPPClient:
 
         member = party.members.get(user_id)
         if member is None:
-            member = party._create_member(body)
-
-            if member.id == self.client.user.id:
-                party._create_clientmember(body)
+            member = (await party._update_members(
+                (body,),
+                remove_missing=False
+            ))[0]
 
         if party.me is not None:
             party.me.do_on_member_join_patch()
@@ -793,14 +945,27 @@ class XMPPClient:
                 member = await self.client.wait_for(
                     'party_member_join',
                     check=check,
-                    timeout=3
+                    timeout=1
                 )
             except asyncio.TimeoutError:
-                if user_id == self.client.user.id:
-                    await party._leave()
-                    p = await self.client._create_party()
-                    self.client.party = p
-                return
+                party_data = await self.client.http.party_lookup(party.id)
+                for m_data in party_data['members']:
+                    if user_id == m_data['account_id']:
+                        member = (await party._update_members(
+                            (m_data,),
+                            remove_missing=False
+                        ))[0]
+                        break
+                else:
+                    if user_id == self.client.user.id:
+                        await party._leave()
+                        p = await self.client._create_party()
+                        self.client.party = p
+                    return
+
+                yielding = party.me._default_config.yield_leadership
+                if party.me and party.me.leader and not yielding:
+                    await party.refresh_squad_assignments()
 
         def _getattr(member, key):
             value = getattr(member, key)
@@ -906,19 +1071,14 @@ class XMPPClient:
         if friend is not None:
             self.client.dispatch_event('party_invite_decline', friend)
 
-    async def process_presence(self, presence: aioxmpp.Presence) -> None:
-        user_id = presence.from_.localpart
-        if user_id == self.client.user.id or not presence.status:
-            return
-
-        if '-' in user_id:
-            return
-
-        if not presence.type_.is_presence_state:
-            return
-
+    @dispatcher.presence()
+    async def process_presence(self, user_id: str,
+                               platform: str,
+                               type_: str,
+                               status: str,
+                               show: str) -> None:
         try:
-            data = json.loads(presence.status.any())
+            data = json.loads(status)
 
             # We do this to filter out kairos from launcher presences
             ch = data.get('bIsEmbedded', False) or data.get('Status', '') != ''
@@ -940,14 +1100,10 @@ class XMPPClient:
             except asyncio.TimeoutError:
                 return
 
-        platform = None
-        if not presence.from_.is_bare:
-            platform = presence.from_.resource.split(':')[2]
-
-        is_available = presence.type_ is aioxmpp.PresenceType.AVAILABLE
+        is_available = type_ is None or type_ == 'available'
 
         try:
-            away = AwayStatus(presence.show)
+            away = AwayStatus(show)
         except ValueError:
             away = AwayStatus.ONLINE
 
@@ -978,10 +1134,7 @@ class XMPPClient:
 
         self.client.dispatch_event('friend_presence', before_pres, _pres)
 
-    def setup_callbacks(self,
-                        messages: bool = True,
-                        process_messages: bool = True,
-                        presences: bool = True) -> None:
+    def setup_callbacks(self, messages: bool = True) -> None:
         message_dispatcher = self.xmpp_client.summon(
             aioxmpp.dispatcher.SimpleMessageDispatcher
         )
@@ -996,30 +1149,6 @@ class XMPPClient:
                 ),
             )
 
-        if process_messages:
-            message_dispatcher.register_callback(
-                aioxmpp.MessageType.NORMAL,
-                None,
-                lambda m: dispatcher.process_event(
-                    self.client,
-                    json.loads(m.body.any())
-                )
-            )
-
-        if presences:
-            presence_dispatcher = self.xmpp_client.summon(
-                aioxmpp.dispatcher.SimplePresenceDispatcher,
-            )
-
-            presence_dispatcher.register_callback(
-                None,
-                None,
-                lambda m: asyncio.ensure_future(
-                    self.process_presence(m),
-                    loop=self.client.loop
-                ),
-            )
-
     async def loop_ping(self) -> None:
         while True:
             await asyncio.sleep(60)
@@ -1059,10 +1188,15 @@ class XMPPClient:
             override_peer=[(
                 self.client.service_domain,
                 self.client.service_port,
-                XMPPOverWebsocketConnector()
+                XMPPOverWebsocketConnector(
+                    self.client,
+                    ws_connector=self.ws_connector
+                )
             )],
             loop=self.client.loop
         )
+        self.xmpp_client.backoff_start = datetime.timedelta(seconds=0.1)
+
         self.muc_service = self.xmpp_client.summon(aioxmpp.MUCClient)
         self.setup_callbacks()
 
@@ -1073,14 +1207,8 @@ class XMPPClient:
         self._ping_task = asyncio.ensure_future(self.loop_ping(),
                                                 loop=self.client.loop)
 
-        asyncio.ensure_future(self.send_presence_on_start(),
-                              loop=self.client.loop)
         self.client.dispatch_event('xmpp_session_establish')
 
-    async def send_presence_on_start(self) -> None:
-        await self.client.wait_until_ready()
-        await self.send_presence(status=self.client.party.last_raw_status)
-
     async def close(self) -> None:
         log.debug('Attempting to close xmpp client')
         if self.xmpp_client.running:
